@@ -115,8 +115,90 @@ SEXP sscCrosstab_L_LI_sizes(SEXP ctab, SEXP permexp)
     UNPROTECT(1);
     return ans;
 }
+
+/** 
+  * Determine the inverse of the permutation of the rows of the sparse,
+  * column-oriented matrix represented by m, n, Ap and Ai that will
+  * concentrate non-zeros in the lower rows.
+  * 
+  * @param m number of rows in the matrix
+  * @param n number of columns in the matrix
+  * @param Ap column pointers in Ai (modified)
+  * @param Ai row indices (modified)
+  * @param rperm on return contains the permutation of the rows
+  * @param cperm if non-null, on return contains the permutation of the columns
+  * @param useL when more than one row matches maxrc, use last match
+  */
+static
+void pair_perm(int m, int n, int Ap[], int Ai[], int rperm[], int cperm[],
+	       int useL)
+{
+    int *cc = Calloc(n, int),	/* column counts */
+	*cm = Calloc(n, int),	/* column removed */
+	*sm = Calloc(m, int),	/* sum of column removals for this row */
+	ii, j, pc,
+	*rc = Calloc(m, int);	/* row counts */
     
-SEXP sscCrosstab_groupedPerm(SEXP ctab)
+    for (j = 0; j < n; j++) {	/* initialize col counts */
+	cc[j] = Ap[j+1] - Ap[j];
+	cm[j] = 0;
+    }
+    pc = 0;
+    for (ii = m - 1; 0 <= ii; ii--) { /* fill rperm from RHS */
+	int maxrc, rr;
+	int i, mincc, p1, p3;
+	
+	mincc = m + 1;		/* find minimum positive cc */
+	for (j = 0; j < n; j++) {
+	    if (0 < cc[j] && cc[j] < mincc) mincc = cc[j];
+	    if (mincc < 2) break;
+	}
+	
+	for (i = 0; i < m; i++) {sm[i] = rc[i] = 0;}
+	for (j = 0; j < n; j++) { /* row counts for cols where cc = mincc */
+	    if (cc[j] == mincc) {
+		int p2 = Ap[j+1];
+		for (i = Ap[j]; i < p2; i++) {
+		    rc[Ai[i]]++;
+		    sm[Ai[i]] += cm[j];
+		}
+	    }
+	}
+	maxrc = -1;		/* find rows with rc[i] == max(rc) */
+	for (i = 0; i < m; i++) {
+	    int ic = rc[i];
+				/* Choose first on row count.  Ties go
+	                         * to smaller sum of moved.  Ties
+	                         * there go to the last one. */
+	    if (ic > maxrc || (ic == maxrc && sm[i] >= sm[rr])) {
+		maxrc = ic;
+		rr = i;
+	    }
+	}
+
+	rperm[rr] = ii;
+
+	p1 = p3 = 0;		/* update cc, Ap and Ai */
+	for (j = 0; j < n; j++) {
+	    int p2 = Ap[j+1];
+	    for (i = Ap[j]; i < p2; i++) {
+		if (Ai[i] == rr) {
+		    cc[j]--; cm[j]++; /* move from count to removed */
+		    if (cperm && cc[j] < 1) cperm[j] = pc++;
+		} else {
+		    if (i != p1) Ai[p1] = Ai[i];
+		    p1++;
+		}
+	    }
+	    Ap[j] = p3;
+	    p3 = p1;		/* save current pos for next iteration */
+	}
+	Ap[n] = p3;
+    }
+    Free(cc); Free(cm); Free(rc); 
+}
+
+SEXP sscCrosstab_groupedPerm(SEXP ctab, SEXP useLast)
 {
     SEXP
 	GpSlot = GET_SLOT(ctab, Matrix_GpSym),
@@ -125,85 +207,41 @@ SEXP sscCrosstab_groupedPerm(SEXP ctab)
     int *Ai = INTEGER(iSlot),
 	*Ap = INTEGER(pSlot),
 	*Gp = INTEGER(GpSlot),
-	nl1 = Gp[1],		/* number of levels of first factor */
-	*icounts,		/* counts for rows */
-	*jcounts = Calloc(nl1, int), /* counts for columns */
-	nl2,			/* delay initializing until nf is known */
-	ii, j, jj,
+	useL = asLogical(useLast),
+	i,
 	n = length(pSlot) - 1,	/* number of columns */
 	nf = length(GpSlot) - 1, /* number of factors */
-	nz = length(iSlot),	/* number of non-zeros */
-	nod = nz - n,		/* number of off-diagonals */
-	*np = Calloc(nl1 + 1, int), /* new array pointers */
-	*ni = Calloc(nod, int),	/* new array row indices */
-	p1, p3;
+	*np = Calloc(n + 1, int), /* column pointers */
+	*ni = Calloc(length(iSlot) - n, int); /* row indices */
     SEXP ans = PROTECT(allocVector(INTSXP, n));
-    int *perm = INTEGER(ans);
 
     if (toupper(*CHAR(STRING_ELT(GET_SLOT(ctab, Matrix_uploSym), 0))) != 'L')
 	error("Lower triangle required in sscCrosstab object");
 
-    for (j = 0; j < n; j++) perm[j] = j; /* initialize permutation to identity */
-
-    if (nf > 1) {
-	nl2 = Gp[2];
-	icounts = Calloc(nl2 - nl1, int);
-	np[0] = 0;
-	for (j = 0; j < nl1; j++) jcounts[j] = 0;
-	p1 = 0;			/* copy off-diagonals from first nl1 cols */
-	for (j = 0; j < nl1; j++) { /* and nl1 <= row < nl2 */
-	    int p3 = Ap[j + 1];
-	    for (jj = Ap[j]; jj < p3; jj++) {
-		int i = Ai[jj];
-		if (nl1 <= i && i < nl2) {
-		    ni[p1++] = i - nl1;	
-		    jcounts[j]++;
-		}
-	    }
-	    np[j+1] = p1;
-	}
-
-	for (ii = 1; ii <= nl2; ii++) {
-	    int maxrc, rr;
-	    int i, minjc = nl2+1;	/* find minimum positive jcount */
-	    for (j = 0; j < nl1; j++) {
-		if (jcounts[i] > 0 && jcounts[j] < minjc) minjc = jcounts[j];
-	    }
-				/* accumulate the row counts on cols where jcount=minjc */
-	    for (i = 0; i < nl2; i++) icounts[i] = 0;
-	    for (j = 0; j < nl1; j++) {
-		if (jcounts[j] == minjc) {
-		    int p2 = np[j+1];
-		    for (i = np[j]; i < p2; i++) icounts[ni[i]]++;
-		}
-	    }
-				/* find the last row whose count in max(icount) */
-	    maxrc = -1;
-	    for (i = 0; i < nl2; i++) {
-		int ic = icounts[i];
-		if (ic >= maxrc) {
-		    maxrc = ic;
-		    rr = i;
-		}
-	    }
-	    perm[nl1 + nl2 - ii] = rr;
-				/* update icounts, np and ni */
-	    p1 = p3 = 0; 
-	    for (j = 0; j < nl1; j++) {
-		int p2 = np[j+1];
-		for (i = np[j]; i < p2; i++) {
-		    if (ni[i] != rr) {
-			if (i != p1) ni[p1] = ni[i];
-			p1++;
-		    }
-		}
-		np[j] = p3;
-		p3 = p1;	/* save the count */
-	    }
-	}
-	Free(icounts);
+    for (i = 0; i < n; i++) {
+	INTEGER(ans)[i] = i;    /* initialize permutation to identity */
     }
-    Free(np); Free(ni); Free(jcounts);
+    np[0] = 0;
+
+    for (i = 1; i < nf; i++) {	/* adjacent pairs of grouping factors */
+	int j, k, p0 = 0, p1 = Gp[i-1], p2 = Gp[i], p3 = Gp[i+1];
+	
+	for (j = p1; j < p2; j++) { /* for this set of columns */
+	    int lk = Ap[j+1];
+	    for (k = Ap[j]; k < lk; k++) {
+		int ii = Ai[k];
+		if (p2 <= ii && ii < p3) { /* check the row */
+		    ni[p0++] = ii - p2;
+		}
+	    }
+	    np[j + 1 - p1] = p0;
+	}
+	pair_perm(p3 - p2, p2 - p1, np, ni,
+		  INTEGER(ans) + p2, INTEGER(ans), useL);
+	for (j = p2; j < p3; j++) INTEGER(ans)[j] += p2;
+    }
+
+    Free(np); Free(ni);
     UNPROTECT(1);
     return ans;
 }
