@@ -2,7 +2,9 @@
 #define MATRIX_SSCCROSSTAB_H
 
 #include "Mutils.h"
+#include "ldl.h"
 
 SEXP sscCrosstab(SEXP flist, SEXP upper);
+SEXP sscCrosstab_L_LI_sizes(SEXP ctab, SEXP permexp);
 
 #endif
